Struct disklru::Store

source ·
pub struct Store<K, V>where
K: Serialize + DeserializeOwned + Eq,
V: Serialize + DeserializeOwned,
{ /* private fields */ }
LRU store, backed by by sled.

The typical use case is: you want a persistent cache, with an interface that has similarities with a basic HashMap. And, you do not want it to grow too much, and prefer to drop on the floor data which has not been accessed for a long time.

LRU is normally used for in-memory caches, but here it’s implemented on something which is persistent. Implementation is inspired from work on the in-memory cache HashLRU.

In most cases this implementation tries to be as close as possible to the standard collections HashMap however there are a few differences:

  • keys and values must be (de)serialiable
  • keys must implement Eq
  • no iterator on mutables
  • as it relies on external data, any operation can fail
  • inserts take references, gets return owned values

This latest aspect is really the fundamental difference: as here all the data is serialized and stored within the store, returning a reference does not really make any sense. Conversely, when putting values in the store, you don’t need to transfer ownership: the store is going to make a serializable copy anyway.

Examples

use disklru::Store;

let mut store = Store::open_temporary(4).unwrap();
store.insert(&1, &10).unwrap();
store.insert(&2, &20).unwrap();
store.insert(&3, &30).unwrap();
store.insert(&4, &40).unwrap();
store.insert(&5, &50).unwrap();
// key1 has been dropped, size is limited to 4
assert_eq!(Some(2), store.lru().unwrap());
assert_eq!(Some(20), store.get(&2).unwrap());
// getting key2 has made key3 the least recently used item
assert_eq!(Some(3), store.lru().unwrap());
assert_eq!(Some(40), store.get(&4).unwrap());
// getting key4 makes it the most recently used item
assert_eq!("[3: 30, 5: 50, 2: 20, 4: 40]", format!("{}", store));
store.flush().unwrap(); // commit

Implementations

source

impl<K, V> Store<K, V>where
K: Serialize + DeserializeOwned + Eq,
V: Serialize + DeserializeOwned,

source

pub fn open_with_config(config: Config, capacity: usize) -> Result<Self>

source

pub fn open_with_path<P: AsRef<Path>>(path: P, capacity: usize) -> Result<Self>

source

pub fn open_temporary(capacity: usize) -> Result<Self>

source

pub fn capacity(&self) -> usize

source

pub fn len(&self) -> usize

source

pub fn resize(&mut self, capacity: usize) -> Result<usize>

source

pub fn clear(&mut self) -> Result<()>

source

pub fn flush(&self) -> Result<usize>

source

pub async fn flush_async(&self) -> Result<usize>

source

pub fn mru(&self) -> Result<Option<K>>

source

pub fn lru(&self) -> Result<Option<K>>

source

pub fn insert(&mut self, k: &K, v: &V) -> Result<Option<V>>

source

pub fn push(&mut self, k: &K, v: &V) -> Result<Option<(K, V)>>

source

pub fn remove(&mut self, k: &K) -> Result<Option<V>>

source

pub fn pop(&mut self, k: &K) -> Result<Option<(K, V)>>

source

pub fn pop_lru(&mut self) -> Result<Option<(K, V)>>

source

pub fn pop_mru(&mut self) -> Result<Option<(K, V)>>

source

pub fn forget(&mut self, k: &K) -> Result<()>

source

pub fn forget_mru(&mut self) -> Result<()>

source

pub fn forget_lru(&mut self) -> Result<()>

source

pub fn contains_key(&mut self, k: &K) -> Result<bool>

source

pub fn bump(&mut self, k: &K) -> Result<()>

source

pub fn get(&mut self, k: &K) -> Result<Option<V>>

source

pub fn get_key_value(&mut self, k: &K) -> Result<Option<(K, V)>>

source

pub fn get_lru(&mut self) -> Result<Option<V>>

source

pub fn peek(&self, k: &K) -> Result<Option<V>>

source

pub fn peek_mru(&self) -> Result<Option<V>>

source

pub fn peek_lru(&self) -> Result<Option<V>>

source

pub fn peek_key_value(&self, k: &K) -> Result<Option<(K, V)>>

source

pub fn export(&self) -> Export<'_, K, V>

source

pub fn iter(&self) -> Iter<'_, K, V>

source

pub fn to_vec(&self) -> Result<Vec<(K, V)>>

source

pub fn import<I>(&mut self, iter: I) -> Result<usize>where
I: Iterator<Item = (K, V)>,

source

pub fn import_map_iter<'a, I>(&mut self, iter: I) -> Result<usize>where
K: Serialize + DeserializeOwned + Eq + 'a,
V: Serialize + DeserializeOwned + 'a,
I: Iterator<Item = (&'a K, &'a V)>,

source

pub fn import_vec_iter<'a, I>(&mut self, iter: I) -> Result<usize>where
K: Serialize + DeserializeOwned + Eq + 'a,
V: Serialize + DeserializeOwned + 'a,
I: Iterator<Item = &'a (K, V)>,

source

pub fn keys(&self) -> Keys<'_, K, V>

source

pub fn values(&self) -> Values<'_, K, V>

source

pub fn export_keys(&self) -> ExportKeys<'_, K, V>

source

pub fn export_values(&self) -> ExportValues<'_, K, V>

source

pub fn dump(&self) -> Result<Dump<K, V>>

source

pub fn restore(&mut self, dump: &Dump<K, V>) -> Result<usize>

source

impl<K, V> Store<K, V>where
K: Serialize + DeserializeOwned + Eq + Hash,
V: Serialize + DeserializeOwned,

source

pub fn to_map(&self) -> Result<HashMap<K, V>>

source

impl<K, V> Store<K, V>where
K: Serialize + DeserializeOwned + Eq + Clone + Debug,
V: Serialize + DeserializeOwned,

source

pub fn audit(&self) -> Result<()>